package fuzzyMaghribis;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import org.nzdis.micro.MicroMessage;
import org.nzdis.micro.inspector.annotations.Inspect;
import org.nzdis.micro.messaging.MTRuntime;
import org.sofosim.environment.GridSim;
import org.sofosim.environment.memoryTypes.AssociativeMemoryInterface;
import org.sofosim.tags.Tag;
import sim.engine.SimState;


public class Agent extends AbstractMaghribi {

	@Inspect
	public Float cheatingProb = null;
	
	@Inspect
	public Float minCheatingRatio = MaghribiSim.MINIMAL_CHEATING_RATIO;
	@Inspect
	public Float maxCheatingRatio = MaghribiSim.MAXIMAL_CHEATING_RATIO;
	
	/**
	 * Queue with trader name as key and goods value as value.
	 */
	@Inspect
	public LinkedHashMap<String, Float> goodsQueue = new LinkedHashMap<String, Float>();
	
	/**
	 * Indicates if agent will ever cheat
	 */
	@Inspect
	public Boolean cheater = null;
	
	@Inspect
	private String finishingTradeWith = null;
	
	public ArrayList<String> pastEmployers = new ArrayList<String>();
	
	@Inspect
	public String getPastEmployers(){
		return pastEmployers.size() + ": " + pastEmployers.toString();
	}
	
	@Inspect
	private AssociativeMemoryInterface cheatingSuccessMemory = null;
	
	public Agent(GridSim sim, String stereotype){
		super(sim);
	}

	@Override
	public synchronized void singleStep(SimState state) {

		if(cheater == null){
			cheater = MaghribiSim.twist.nextBoolean(MaghribiSim.QUOTA_OF_CHEATERS);//sim.random.nextBoolean(MaghribiSim.QUOTA_OF_CHEATERS);
		}
		if(cheatingProb == null){
			if(MaghribiSim.MINIMAL_CHEATING_PROB.equals(MaghribiSim.MAXIMAL_CHEATING_PROB)){
				cheatingProb = MaghribiSim.MAXIMAL_CHEATING_PROB;
			}
			while(cheatingProb == null){
				cheatingProb = new Float(sim.random().nextInt(100))/100;
				cheatingProb = Math.min(MaghribiSim.MAXIMAL_CHEATING_PROB, cheatingProb);
				cheatingProb = Math.max(MaghribiSim.MINIMAL_CHEATING_PROB, cheatingProb);
			}
			print("Cheating probability: " + cheatingProb);

			sim.random().nextInt();
			sim.random().nextInt();
		}
		
		//Communication & Negotiation, if asynchronous message passing
		if(!MTRuntime.isSynchronousOperationMode() && haveUnprocessedMessages()){
			while(haveUnprocessedMessages()){
				handleMessage(getMessage());
			}
		}
		//Action
		
		if(!goodsQueue.isEmpty()){
			//take first good and handle
			String trader = goodsQueue.keySet().iterator().next();
			Float goodsVal = goodsQueue.get(trader);
			
			float value = 0f;
			//market achievement
			value = goodsVal;
			//how much change?
			Double change = null;
			while(change == null || change < 0.0 || change > MaghribiSim.MAXIMAL_MARKET_WIN_LOSS){
				change = new Double(sim.random().nextInt(100))/100;
			}
			print("Market change: " + change + " is ...");
			//win or loss
			boolean win = true;
			StringBuilder buf = new StringBuilder();
			if(sim.random().nextBoolean(MaghribiSim.MARKET_WIN_PROBABILITY)){
				value += goodsVal * change;
				buf.append("Gain: ");
				win = true;
			} else {
				value -= goodsVal * change;
				buf.append("Loss: ");
				win= false;
			}
			print(buf.append("Goods value on market: ").append(value));
			//switch to identify if agent has cheated when generating reporting message
			boolean cheated = false;
			
			Float ownGain = 0f;
			//checking if I should cheat - depending on me - am I cheater, and the probability
			//only cheat when achieving market gain
			if(MaghribiSim.cheating && cheater){
				boolean reallyCheating = sim.random().nextBoolean(cheatingProb);
				
				if(reallyCheating){
					//cheating
					//calculate random value
					Float ratio = Float.valueOf(sim.random().nextInt(100))/100;
					//if maximum cheating ratio is set, consider that
					if(maxCheatingRatio != null && minCheatingRatio != null){
						if(maxCheatingRatio.equals(minCheatingRatio)){
							ratio = maxCheatingRatio;
						} else {
							while(ratio > maxCheatingRatio && ratio < minCheatingRatio){
								ratio = Float.valueOf(sim.random().nextInt(100))/100;
							}
						}
					}
					
					//calculate own gain (by cheating) from total value realized at market
					ownGain = ratio * value;
					value -= ownGain;
					print("Cheated " + trader + " and reporting achieved " + value + " for good value " + goodsVal + "; own gain from cheating: " + ownGain);
					Stats.cheated();
					cheated = true;
				}
			} else {
				if(cheater && !win){
					print("Did not cheat as no market gain.... Would have otherwise....");
				}
			}
			String target = trader;
			MicroMessage message = null;
			
			message = MessageFactory.makeTradeConfirmation(target, value, "", cheated, ownGain);
			//indicate that you are employed
			Stats.haveOneOffEmployment();
			if(cheater){
				//only called to indicate how many cheaters (although not necessarily cheating at this instance) are under employment
				Stats.haveOneOffEmploymentAndAmCheaterByNature(this);
			}
			if(finishingTradeWith != null){
				finishingTradeWith = null;
			}
			
			//first remove goods from memory
			goodsQueue.remove(trader);
			//then send message - if other way around, trader will receive goods twice!
			send(message);
		}
	}
	
	@Override
	protected void initialize() {
		semaphore.setName("Message queue for agent " + me());
	}

	public void handleOneOffMessage(MicroMessage message){
		if(MessageFactory.isGoodsTransferMessage(message)){
			goodsQueue.put(message.getSender(), Float.parseFloat(message.get(MessageFactory.GOODS_VALUE).toString()));
		} else {
			printError("Am in one-off mode and did not receive goods transfer message.");
		}
	}

	@Override
	public void handleMessage(MicroMessage message) {
		
		if(MaghribiSim.oneOffTrades){
			handleOneOffMessage(message);
			return;
		}
		
	}

	@Override
	public Float getPlaneWeight(String plane) {
		return 0f;
	}

	@Override
	public Float getTagWeight(Tag tag) {
		return 0f;
	}
	
}
