package fuzzyMaghribis;

import java.util.ArrayList;
import java.util.HashMap;
import org.apache.commons.math3.distribution.NormalDistribution;
import org.nzdis.micro.SystemAgentLoader;
import org.sofosim.clustering.VertexPoint3D;
import org.sofosim.environment.GridSim;
import org.sofosim.environment.annotations.SimulationParam;
import org.sofosim.graph.initializers.ForceGraphInitializer;
import sim.engine.RandomSequence;
import sim.engine.Steppable;
import ec.util.MersenneTwisterFast;
import fuzzyMaghribis.network.GraphGenerators;
import fuzzyMaghribis.network.GraphMeasures;
import fuzzyMaghribis.network.Network;

public class MaghribiSim extends GridSim{

	/** Constants */
	
	/** Simulation-wide prefixes for Trader and agent names */
	public static final String TRADER_PREFIX = "Trader_";
	public static final String AGENT_PREFIX = "Agent_";
	public static final String INSTITUTION = "MERCHANT_COURT";
	
	public static final String AGENT_RANDOM = "Randomly populated Agent";
	public static final String TRADER_RANDOM = "Randomly populated Trader";
	
	
	public static Integer DIMENSION = 600;
	
	/** Simulation initialization parameters */

	@SimulationParam
	public static Integer NO_OF_TRADERS = 150;
	@SimulationParam
	public static Integer NO_OF_AGENTS = 200;
	
	
	
	/**
	 * Probability of employers to employ agents. If set to null here, it is determined individually
	 */
	@SimulationParam
	public static final Float EMPLOYMENT_PROBABILITY = 1.0f;
	
	/** Cheater-related */
	/**
	 * Indicates the quota of cheaters; agents are cheaters or non-cheaters (never cheat), probs. for cheater are below
	 */
	@SimulationParam
	public static Float QUOTA_OF_CHEATERS = 0.4f;
	/**
	 * If they are cheaters, what is their minimal cheating probability?
	 */
	@SimulationParam
	public static Float MINIMAL_CHEATING_PROB = 0.2f;
	/**
	 * If they are cheaters, what is their maximal cheating probability?
	 */
	@SimulationParam
	public static Float MAXIMAL_CHEATING_PROB = 0.8f;
	/**
	 * how much do agents cheat min. if they cheat (ratio of goods value after market win/loss)
	 */
	@SimulationParam
	public static final Float MINIMAL_CHEATING_RATIO = 0.3f;
	/**
	 * how much do they cheat about max. if they cheat (ratio of goods value after market win/loss)
	 */
	@SimulationParam
	public static final Float MAXIMAL_CHEATING_RATIO = 0.9f;
	
	
	public static MersenneTwisterFast twist = null;
	
	public static NormalDistribution ndist; 
	
	/** 
	 * Ratios for the next three parameters need sum to 1.0
	 * CHEATING_CORRECT_DETECTION_PROBABILITY + CHEATING_SUSPICION_PROBABILITY + CHEATING_FALSE_DETECTION_PROBABILITY = 1.0
	 */
	
	/**
	 * Probability for correct detection of cheating
	 */
	@SimulationParam
	public static Float CHEATING_CORRECT_DETECTION_PROBABILITY = 1f;

	/**
	 * Probability for suspicion detection
	 */
	@SimulationParam
	public static Float CHEATING_SUSPICION_PROBABILITY = 0.0f;
	
	/**
	 * Probability for false detection of cheating
	 */
	@SimulationParam
	public static Float CHEATING_FALSE_DETECTION_PROBABILITY = 0.0f;
	
	
	
	/**
	 * Quota of traders addressed by a cheater announcement (if {@link #useTraderInformationNetworkInsteadOfFuzziness} is false)
	 */
	@SimulationParam
	public static Float COMMUNICATION_FUZZINESS_QUOTA = 0.1f;
	
	/** Cheater memory handling related */
	
	/**
	 * Discount value for cheater valuation in memory
	 * (trade-off: high discount factor means I do not forget fast and stay aware of cheaters, low means I will reemploy cheaters quicker) 
	 */
	@SimulationParam
	public static Float CHEATER_MEMORY_DISCOUNT_FACTOR = 0.99f;
	
	/**
	 * Valuation of others' cheater experience in memory
	 */
	@SimulationParam
	public static final Float CHEATER_VALENCE_OTHERS = -1.0f;
	
	/**
	 * Valuation of own cheater experiences in memory
	 */
	@SimulationParam
	public static final Float CHEATER_VALENCE_OWN = CHEATER_VALENCE_OTHERS * 2;
	
	/**
	 * positive reinforcement valuation - lower than in negative case, positive is not as significant
	 */
	@SimulationParam
	public static final Float POSITIVE_VALENCE_OWN = CHEATER_VALENCE_OWN * 0.25f * -1;
	
	/**
	 * positive reinforcement valuation if reported by others - lower than in negative case, positive is not as significant, and lower than own experience
	 */
	@SimulationParam
	public static final Float POSITIVE_VALENCE_OTHERS = POSITIVE_VALENCE_OWN * 0.5f;

	
	/**
	 * Threshold for deletion of memory entries, abs. value above and below zero
	 */
	@SimulationParam
	public static Float MEMORY_ENTRY_DELETION_THRESHOLD_ABS_VAL_FROM_ZERO = 0.00f;
	
	
	/** Market parameters */
	
	/**
	 * Probability realize a gain in market transactions
	 */
	@SimulationParam
	public static final Float MARKET_WIN_PROBABILITY = 0.8f;
	
	/**
	 * Max. win or loss in market transactions
	 */
	@SimulationParam
	public static final Float MAXIMAL_MARKET_WIN_LOSS = 0.3f;
	
	/**
	 * Goods value
	 */
	@SimulationParam
	public static final Float GOODS_VALUE = 200.0f;
	
	/**
	 * Number of traders selected for initial contact (sharing of information about agents)
	 */
	@SimulationParam
	public static final Integer INITIAL_TRADER_SELECTION = 10;
	
	
	/**
	 * Static ratio of reputation specified at simulation start
	 */
	@SimulationParam
	public static final Float REPUTATION_BASE_RATIO = 0.5f;
	
	
	/** Switches */
	
	/**
	 * Cheating activated?
	 */
	@SimulationParam
	public static boolean cheating = true;
	
	/**
	 * Cheating discovery by traders
	 */
	@SimulationParam
	public static boolean cheatingDiscovery = true;
	
	/**
	 * Indicates if cheating detection includes false positives if cheating detection is activated (i.e. non-cheaters detected as cheaters) or only false negatives (missed out on detected cheaters)
	 */
	@SimulationParam
	public static boolean allowFalsePositiveCheatingDetection = false;

	/**
	 * Cheating memory
	 */
	@SimulationParam
	public static boolean cheatingMemory = true;
	
	/**
	 * Traders forgetting about cheaters
	 */
	@SimulationParam
	public static boolean forgettingCheaters = true;
	
	/**
	 * Traders communicate about cheaters, i.e. collectivist mode of information sharing
	 */
	@SimulationParam
	public static boolean cheatingCommunication = false;
	
	
	public static final boolean oneOffTrades = true;
	
	/**
	 * Maximum times a trader can pick a candidate randomly in one round 
	 * before he falls back to picking one he has the best memory of.
	 */
	@SimulationParam
	public static Integer maxCandidatePicksBeforePickingHighestPossible = 5;
	
	/**
	 * Indicates the maximum number of advice requests from other 
	 * traders before sending goods to seller. If not successful, 
	 * random pick of seller and sending goods.
	 */
	@SimulationParam
	public static Integer maxAdviceRequestsForOneOffTrades = 5;
	
	/**
	 * Uses discrete memory that maintains entries on FIFO basis 
	 * and maintains a consistent max length (see cheaterMemoryLength).
	 */
	@SimulationParam
	public static boolean useDiscreteMemoryInsteadOfDiscountMemory = true;
	
	/**
	 * Length of memory for discrete memory version
	 */
	@SimulationParam
	public static Integer cheaterMemoryLengthForDiscreteMemory = 30;

	/**
	 * Information network structure
	 */
	@SimulationParam
	public static String traderInfoNetworkType = NetworkType.NETWORK_RANDOM;
	
	/**
	 * Probability for rewiring nodes if Watts-Strogatz network generator is used.
	 */
	@SimulationParam
	public static Float wattsStrogatzRewiringProbability = 0.3f;
	
	/**
	 * Holds trader information network instance.
	 */
	public static Network traderInfoNetwork = null;
	
	/**
	 * Holds reference to frame showing graph measures.
	 */
	private static GraphMeasures graphMeasures = null;
	
	/**
	 * Maximum number of fixed advisor relations (if useFixedTraderRelations is activated).
	 */
	@SimulationParam
	public static Integer maxNumberOfTraderTrustRelations = 10; 
	
	/**
	 * Indicates if traders seek advice from other traders before 
	 * engaging in one-off employments.
	 */
	@SimulationParam
	public static boolean takeAdviceFromTraders = oneOffTrades && false;
	
	/**
	 * Probability to be truthful when sharing information about 
	 * cheaters
	 */
	@SimulationParam
	public static Float adviseTruthfullyProbability = 0.7f;
	
	/**
	 * Minimal probability of advising truthfully - can not be 
	 * overridden by dynamic adjustment of advice probabilities
	 */
	@SimulationParam
	public static Float minAdviseTruthfullyProbability = 0.05f;
	
	/**
	 * Probability for testing given advice by employing nevertheless for 
	 * one round
	 */
	@SimulationParam
	public static Float adviceTestProbability = 0.3f;
	
	/**
	 * Minimal advice test probability - even with dynamic adjustment of 
	 * test probability this value will never be overridden.
	 */
	@SimulationParam
	public static Float minAdviceTestProbability = 0.05f;
	
	/**
	 * Indicates if advice probabilities are dynamically adjusted based on experience.
	 */
	@SimulationParam
	public static boolean adjustAdviceProbabilities = true;
	
	/**
	 * Steps of adjustment for advice probabilities
	 */
	@SimulationParam
	public static Float adviceAdjustmentStep = 0.005f;
	
	/**
	 * Switch to activate increased truthful reporting for known traders
	 */
	@SimulationParam
	public static boolean increasedTrustProbabilityForTraderInTraderConnections = true;
	
	/**
	 * Probability by which truthful reporting probability is increased when advising known fellow traders 
	 * (increasedTrustProbabilityForTradersInTraderConnections).
	 */
	@SimulationParam
	public static Float trustIncreaseForKnownTraders = 0.1f;
	
	/**
	 * Provides truthful advice to other traders that have themselves advised positively in the past
	 */
	@SimulationParam
	public static boolean correctAdviceToPositivelyRememberedAdvisors = true;
	
	/**
	 * remember successful interactions
	 */
	@SimulationParam
	public static boolean rememberPositiveTransactions = true;
	
	/**
	 * Indicator if positive transactions are also communicated to acquainted traders - only works if cheatingCommunication is activated
	 */
	@SimulationParam
	public static boolean communicatePositiveTransactions = cheatingCommunication && rememberPositiveTransactions && true;
	
	/**
	 * Indicates if a trader information network is used instead of fuzzy random information of other traders about cheaters
	 */
	@SimulationParam
	public static boolean useTraderInformationNetworkInsteadOfFuzziness = cheatingCommunication && true;
	
	/**
	 * Indicates if traders inform each other reciprocally if using TraderInformationNetwork (see previous parameter) (i.e. sender of cheater information will be informed by me in the future)
	 */
	@SimulationParam
	public static boolean reciprocalTraderInformationNetwork = cheatingCommunication && useTraderInformationNetworkInsteadOfFuzziness && true;
	
	/**
	 * Initialization of trader information network as scale free, else as random selection of targets (i.e. random network)
	 */
	@SimulationParam
	public static boolean initializeTraderInfoNetworkScaleFree = cheatingCommunication && useTraderInformationNetworkInsteadOfFuzziness && false;
	
	/**
	 * Means to keep track of results subfolder in simulation parameter set
	 */
	@SimulationParam
	public static String globalSubfolder = null;
	
	public MaghribiSim(long seed) {
		this(seed, true);
	}
	
	/**
	 * Extended constructor to deactivate starting of UI controller
	 * @param seed
	 * @param startGraphController
	 */
	public MaghribiSim(long seed, boolean startGraphController) {
		super(false, DIMENSION, DIMENSION, DIMENSION, true, false, new Stats(startGraphController, false, 1), seed);
		
		twist = new MersenneTwisterFast(seed);
	}
	
	private boolean validateAssumptions(){
		StringBuffer assumptions = new StringBuffer("Invalid Assumptions: ");
		boolean failed = false;
		System.err.println("Corr and Incorr probs: " + CHEATING_CORRECT_DETECTION_PROBABILITY + ", " + CHEATING_FALSE_DETECTION_PROBABILITY);
		if(CHEATING_CORRECT_DETECTION_PROBABILITY + CHEATING_FALSE_DETECTION_PROBABILITY != 1.0){
			assumptions.append("correct cheating detection probability and wrong detection don't add up to 1.0 (no suspicion support);");
			failed = true;
		}
		if(failed){
			System.err.println(assumptions);
		}
		return !failed;
	}
	
	public static HashMap<String, Trader> traderMap = new HashMap<String, Trader>();
	public static HashMap<String, Agent> agentMap = new HashMap<String, Agent>();
	
	@Override
	public void start() {
		if(!validateAssumptions()){
			throw new RuntimeException("Not all assumptions have been satisfied.");
		}
		super.start();
		
		globalSubfolder = Stats.getGlobalSubfolderForOutput();

		
		//start stats (will register itself with scheduler)
		getStatistics().step(this);
		
		//Traders
		Trader firstTrader = null;
		for(int x=0; x<NO_OF_TRADERS; x++){

			Trader role = new Trader(this, TRADER_RANDOM);
			if(x == 0){
				firstTrader = role;
				SystemAgentLoader.newAgent(firstTrader, TRADER_PREFIX + x);
			} else {
				firstTrader.getAgent().getGroup().getAgentLoader().newAgent(role, TRADER_PREFIX + x);
			}
			agentDirectory.put(role.me(), role);
			traderMap.put(role.me(), role);
			//graph handling
			if(graphHandler != null){
				graphHandler.addVertex(ForceGraphInitializer.FORCES_GRAPH, new VertexPoint3D(role.me()));
			}
		}
		//Agents
		Agent firstAgent = null;
		for(int x=0; x<NO_OF_AGENTS; x++){
			Agent role = new Agent(this, AGENT_RANDOM);
			if(x == 0){
				firstAgent = role;
				SystemAgentLoader.newAgent(firstAgent, AGENT_PREFIX + x);
			} else {
				firstAgent.getAgent().getGroup().getAgentLoader().newAgent(role, AGENT_PREFIX + x);
			}
			agentDirectory.put(role.me(), role);
			agentMap.put(role.me(), role);
			if(graphHandler != null){
				graphHandler.addVertex(ForceGraphInitializer.FORCES_GRAPH, new VertexPoint3D(role.me()));
			}
			
		}
		
		//RANDOM SCHEDULING
		//both traders and agents initialized, now only scheduling leSteppableomized
		ArrayList<Steppable> allAgents = new ArrayList<Steppable>();
		allAgents.addAll(traderMap.values());
		allAgents.addAll(agentMap.values());
		//ensure it synchronizes on the RNG
		RandomSequence randomSequence = new RandomSequence(allAgents.toArray(new Steppable[allAgents.size()]), true);
		schedule.scheduleRepeating(randomSequence);
		
		
		//preassign connections
		if(MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_WATTS_STROGATZ)){
			GraphGenerators.generateWattsStrogatzGraph(maxNumberOfTraderTrustRelations, seed(), false);
		} else if(MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_BARABASI_ALBERT)){
			GraphGenerators.generateBarabasiAlbertGraph(maxNumberOfTraderTrustRelations, seed(), false);
		}
		
		if(traderInfoNetworkType != null && !traderInfoNetworkType.isEmpty()){
			//take network of first accessible agent - hope that works :)
			traderInfoNetwork = traderMap.values().iterator().next().traderInfoNetwork;
			if(SHOW_STATS_GRAPHS){
				graphMeasures = new GraphMeasures(traderInfoNetwork);
				graphMeasures.setVisible(true);
			}
		}
	}
	
	@Override
	public void finish(){
		super.finish();
		if(traderInfoNetwork != null){
			if(graphMeasures != null){
				graphMeasures.dispose();
			}
			traderInfoNetwork.reset();
		}
	}
	
	
	
	public static void main(String args[]){
		doLoop(MaghribiSim.class, args);
	}

}
