package fuzzyMaghribis;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import org.nzdis.micro.MTConnector;
import org.nzdis.micro.MessageFields;
import org.nzdis.micro.MicroMessage;
import org.nzdis.micro.SystemAgentLoader;
import org.nzdis.micro.inspector.annotations.CollectiveView;
import org.nzdis.micro.inspector.annotations.Inspect;
import org.sofosim.environment.GridSim;
import org.sofosim.environment.memoryTypes.AssociativeMemoryInterface;
import org.sofosim.environment.memoryTypes.DiscountingMemory;
import org.sofosim.environment.memoryTypes.DiscreteAggregatingMemory;
import org.sofosim.environment.stats.StatsCalculator;
import org.sofosim.tags.Tag;
import fuzzyMaghribis.network.Advice;
import fuzzyMaghribis.network.Advisor;
import fuzzyMaghribis.network.FixedNetwork;
import fuzzyMaghribis.network.Network;
import fuzzyMaghribis.network.RandomNetwork;
import fuzzyMaghribis.network.GeneratedNetwork;
import sim.engine.SimState;

public class Trader extends AbstractMaghribi implements Advisor {

	@Inspect
	/* probability that the trader will employ an agent */
	public Float employmentProb = MaghribiSim.EMPLOYMENT_PROBABILITY;
	
	public boolean useTraderInformationNetwork = MaghribiSim.useTraderInformationNetworkInsteadOfFuzziness;
	
	/**
	 * Maintains memory about cheaters.
	 */
	@Inspect
	@CollectiveView
	public AssociativeMemoryInterface cheaterMemory = (MaghribiSim.useDiscreteMemoryInsteadOfDiscountMemory ? new DiscreteAggregatingMemory(MaghribiSim.cheaterMemoryLengthForDiscreteMemory) : new DiscountingMemory(MaghribiSim.CHEATER_MEMORY_DISCOUNT_FACTOR, ""));
	
	/**
	 * Memory for profit caused by a given seller
	 */
	@Inspect
	public AssociativeMemoryInterface profitMemory = (MaghribiSim.useDiscreteMemoryInsteadOfDiscountMemory ? new DiscreteAggregatingMemory(MaghribiSim.cheaterMemoryLengthForDiscreteMemory) : new DiscountingMemory(MaghribiSim.CHEATER_MEMORY_DISCOUNT_FACTOR, ""));
	
	/** 
	 * Storing traders I am acquainted to and share information with 
	 */
	@Inspect
	public ArrayList<String> traderConnections = new ArrayList<String>();
		
	/**
	 * Most negative cheating extent (not negative profit) - pure cheating extent by seller
	 */
	@Inspect
	Float mostNegativeCheating = 0.0f;
	
	public Trader(GridSim sim, String stereotype){
		super(sim);
	}

	
	StatsCalculator clusCalc = null;
	
	ArrayList<String> traderNameList = new ArrayList<String>();
	
	/**
	 * Trader network type
	 */
	Network traderInfoNetwork = (MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_RANDOM) ? new RandomNetwork() : 
			((MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_FIXED) 
					|| MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_FIXED_MUTUAL)) ? new FixedNetwork() : 
			((MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_WATTS_STROGATZ) 
					|| MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_BARABASI_ALBERT)) ? new GeneratedNetwork() : null)));
	
	//Map indicating adviser memory - candidate seller as key, adviser as value
	HashMap<String, Advice> candidateAdvisorMap = new HashMap<String, Advice>();
	//maintains information whether transaction is based on own others' judgement (always maintained)
	HashMap<String, Boolean> transactionBasedOnOwnJudgement = new HashMap<>();
	
	@Inspect
	@CollectiveView
	public AssociativeMemoryInterface advisorMemory = (MaghribiSim.useDiscreteMemoryInsteadOfDiscountMemory ? new DiscreteAggregatingMemory(MaghribiSim.cheaterMemoryLengthForDiscreteMemory) : new DiscountingMemory(MaghribiSim.CHEATER_MEMORY_DISCOUNT_FACTOR, ""));
	
	/**
	 * Individual probability of testing advice
	 */
	@Inspect
	public Float adviceTestProbability = MaghribiSim.adviceTestProbability;
	
	/**
	 * Individual probability of advising truthfully
	 */
	@Inspect
	public Float adviseTruthfullyProbability = null;

	
	/**
	 * Adjusts the probabilities for testing advice and advising truthfully 
	 * depending on experience between extreme values of 0 and 1.
	 * @param adviceExperience Positive or negative experience with advisor
	 */
	private void adjustAdviceProbabilities(Float adviceExperience){
		print("Adjusting advice probabilities based on advice experience " + adviceExperience);
		//adviceExperience = advisorMemory.getMeanOfAllEntries().floatValue();
		if(adviceExperience < 0){
			//increase advice test probability
			if((1f - adviceTestProbability) >= MaghribiSim.adviceAdjustmentStep){
				adviceTestProbability += MaghribiSim.adviceAdjustmentStep;
			}
			//reduce probability for truthfully advising. Do not override lowest probability
			if((adviseTruthfullyProbability - MaghribiSim.minAdviseTruthfullyProbability) >= MaghribiSim.adviceAdjustmentStep){
				adviseTruthfullyProbability -= MaghribiSim.adviceAdjustmentStep;
			}
		} else if(adviceExperience > 0) {
			//reduce probability for testing advice. Do not override lowest test probability
			if((adviceTestProbability - MaghribiSim.minAdviceTestProbability) >= MaghribiSim.adviceAdjustmentStep){
				adviceTestProbability -= MaghribiSim.adviceAdjustmentStep;
			}
			//increase probability for truthful advising
			if((1f - adviseTruthfullyProbability) >= MaghribiSim.adviceAdjustmentStep){
				adviseTruthfullyProbability += MaghribiSim.adviceAdjustmentStep;
			}
		}
	}
	
	@Override
	public synchronized void singleStep(SimState state) {
		
		if(this.adviseTruthfullyProbability == null){
			this.adviseTruthfullyProbability = (float)MaghribiSim.ndist.sample();

			sim.random().nextInt();
			sim.random().nextInt();
		}
		sim.random().nextInt();
		
		
		if(MaghribiSim.cheatingCommunication && useTraderInformationNetwork){
			if(!MaghribiSim.initializeTraderInfoNetworkScaleFree){
				setupInitialTraderConnections(MaghribiSim.INITIAL_TRADER_SELECTION);
			}
			//else the power law distribution will already have been applied beforehand
		}
		if(employmentProb == null){
			employmentProb = sim.random().nextFloat();
			print("Set employment probability to " + employmentProb);
		}
		
		if(traderNameList.isEmpty()){
			for(String traderName: sim.agentDirectory.keySet()){
				if(traderName.startsWith(MaghribiSim.TRADER_PREFIX) && !traderName.equals(me())){
					traderNameList.add(traderName);
				}
			}
			print("TraderName list: (" + traderNameList.size() + "):\n" + traderNameList);
		}
		
		//initialize trader network here - for dynamically generated network types
		if(!traderInfoNetwork.isInitialized()){
			traderInfoNetwork.initialize(this, sim);
		}
		
		
		//SETUP STUFF DONE UP TO HERE
		
		
		String candidate = null;
		Integer candidatePickAttempts = 0;
		int adviceCount = 0;
		while(candidate == null){
			//get random seller 
			candidate = MTConnector.getRandomAgents(1, traderNameList).get(0);
			print("Candidate: " + candidate);
			candidatePickAttempts++;
			boolean candidateChosen = false;
			//if maximal number of pick attempts done and still bad candidate, pick highest possible
			if(candidatePickAttempts >= MaghribiSim.maxCandidatePicksBeforePickingHighestPossible
					&& cheaterMemory.containsKey(candidate) && cheaterMemory.getValueForKey(candidate) < 0){
				candidate = cheaterMemory.getKeyForHighestValue();
				//transactionBasedOnOwnJudgement.put(candidate, true);
				print("Have bad memory of all, picking candidate with best memory: " + candidate + ": " + cheaterMemory.getValueForKey(candidate) + " - ignoring further advice.");
				break;
			}
			sim.random().nextBoolean();
			if(candidate != null && cheaterMemory.containsKey(candidate) 
					&& cheaterMemory.getValueForKey(candidate) < 0){
				//bad experience
				print("Had bad experience with seller " + candidate);
				print("Pick count: " + candidatePickAttempts);
				candidate = null;
			} else if(candidate != null && cheaterMemory.containsKey(candidate)
					&& cheaterMemory.getValueForKey(candidate) > 0){
				//good experience
				print("Had good experience with seller " + candidate + " - will employ this round.");
				transactionBasedOnOwnJudgement.put(candidate, true);
				candidateChosen = true;
			}
			//check if you will be testing advice anyway, even if own positive memory about candidate
			boolean testingAdvice = sim.random().nextBoolean(adviceTestProbability);
			if(candidate != null && (!candidateChosen || (candidateChosen && testingAdvice)) && MaghribiSim.takeAdviceFromTraders){
				//if no candidate chosen based on good own experience - ask for advice
				
				//ask for advice and ensure that advice is complete, not from a disregarded advisor and not from myself - as long as I have free tries
				Advice receivedAdvice = null;
				ArrayList<String> checkedAdvisorsThisRound = new ArrayList<String>();
				while((receivedAdvice == null || !receivedAdvice.successfulResponse() 
						|| receivedAdvice.getAdvisor() == null || receivedAdvice.getAdvisor().equals(me())
						|| checkedAdvisorsThisRound.contains(receivedAdvice.getAdvisor())
						)
					&& adviceCount < MaghribiSim.maxAdviceRequestsForOneOffTrades 
					){
					receivedAdvice = traderInfoNetwork.getAdvice(candidate);
					//memorize advisor for this round - don't reuse - but check for null first
					if(receivedAdvice.getAdvisor() != null && !receivedAdvice.getAdvisor().isEmpty() 
							&& !checkedAdvisorsThisRound.contains(receivedAdvice.getAdvisor())){
						checkedAdvisorsThisRound.add(receivedAdvice.getAdvisor());
						//no successful response
						if(!receivedAdvice.successfulResponse()){
							print("No successful response from " + receivedAdvice.getAdvisor() + " (Attempt: " + adviceCount + ")");
						} else{
							//valid response from an unknown
							break;
						}
						adviceCount++;
					} else {
						if(!traderConnections.isEmpty()){
							//check if all known traders have already been asked - else it ends up looping
							boolean oneTraderOfConnectionsNotQueried = false;
							for(int i = 0; i < traderConnections.size(); i++){
								if(!checkedAdvisorsThisRound.contains(traderConnections.get(i))){
									oneTraderOfConnectionsNotQueried = true;
									break;
								}
							}
							if(!oneTraderOfConnectionsNotQueried){
								//if all acquainted traders have been queried, no further advice opportunity - live with what you have.
								print("Already asked all known traders.");
								break;
							}
						}
						print("Already got advice from advisor " + receivedAdvice.getAdvisor() + " this round.");
						//only increase count if did not query same advisor as before
					}
				}
				
				if(!receivedAdvice.successfulResponse()){
					print("Didn't get advice from trader " + receivedAdvice.getAdvisor() + " (Advice attempt: " + adviceCount + "). Taking seller " + candidate);
					//transactionBasedOnOwnJudgement.put(candidate, true);
				} else if(advisorMemory.containsKey(receivedAdvice.getAdvisor()) && advisorMemory.getValueForKey(receivedAdvice.getAdvisor()) < 0f){
					//negative experience with advisor - ignore and experience with selected seller
					print("Don't want advice from " + receivedAdvice.getAdvisor() + ", will act on my own and picked " + candidate);
					transactionBasedOnOwnJudgement.put(candidate, true);
				} else {
					//no negative experience with advisor
					print("Have advice from " + receivedAdvice.getAdvisor() + " on " + candidate + ": " + receivedAdvice.getAdviceValue());
					//negative advice
					if(receivedAdvice.getAdviceValue() < 0){
						print("Got negative advice on " + candidate + " from " + receivedAdvice.getAdvisor());
						if(testingAdvice){
							print("Testing negative advice and nevertheless employ seller " + candidate);
							//employ candidate and monitor advice
							candidateAdvisorMap.put(candidate, receivedAdvice);
							//suggestion overridden by me - own decision
							transactionBasedOnOwnJudgement.put(candidate, true);
						} else {
							//blindly believe - pick new candidate seller
							print("NOT testing negative advice and nevertheless employ seller " + candidate);
							//memorize as bad experience of others
							cheaterMemory.memorize(candidate, MaghribiSim.CHEATER_VALENCE_OTHERS);
							//memorize as good experience of own (i.e. I believe other trader)
							advisorMemory.memorize(receivedAdvice.getAdvisor(), MaghribiSim.POSITIVE_VALENCE_OWN);
							//reset candidate (so a new one is chosen)
							candidate = null;
						}
					} else {
						//positive advice
						if(testingAdvice){
							//test positive advice
							print("Testing positive advice and employ seller " + candidate);
							//employ candidate and monitor advice
							candidateAdvisorMap.put(candidate, receivedAdvice);
							//recommended by other
							transactionBasedOnOwnJudgement.put(candidate, false);
						} else {
							print("NOT testing positive advice and employ seller " + candidate);
							//do not test and believe that outcome is positive
							//memorize as bad experience of others
							cheaterMemory.memorize(candidate, MaghribiSim.POSITIVE_VALENCE_OWN);
							//memorize as good experience of own (i.e. I believe other trader)
							advisorMemory.memorize(receivedAdvice.getAdvisor(), MaghribiSim.POSITIVE_VALENCE_OWN);
							//and go for it - employ selected seller...
							transactionBasedOnOwnJudgement.put(candidate, false);
						}
					}
				}
			}
		}
		
		print("Reporting number of advice requests: " + adviceCount);
		//Stats about advice requests
		Stats.numberOfAdviceRequestsSent(adviceCount);
		
		//candidate should never be null
		if(candidate == null){
			throw new RuntimeException("Candidate should never be null at this stage.");
		}
		
		Float goodsValue = MaghribiSim.GOODS_VALUE;
		send(MessageFactory.makeSendGoodsOneOffMessage(candidate, goodsValue));
		if(MaghribiSim.forgettingCheaters){
			//discount all reputation values for agents (and delete them if too low)
			cheaterMemory.forgetAtRoundEnd(MaghribiSim.MEMORY_ENTRY_DELETION_THRESHOLD_ABS_VAL_FROM_ZERO);
		}
		advisorMemory.forgetAtRoundEnd(MaghribiSim.MEMORY_ENTRY_DELETION_THRESHOLD_ABS_VAL_FROM_ZERO);
		profitMemory.forgetAtRoundEnd(MaghribiSim.MEMORY_ENTRY_DELETION_THRESHOLD_ABS_VAL_FROM_ZERO);
	}

	@Override
	protected void initialize() {
		//wealth = MaghribiSim.INITIAL_TRADER_WEALTH;
		//print("Initialized as Trader");
		semaphore.setName("Message queue for agent " + me());
		//semaphore.printWaitInformationOutput(false);
		
		if(!traderInfoNetwork.isInitialized()
				&& (!MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_FIXED)
				&& !MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_FIXED_MUTUAL)
				&& !MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_RANDOM))){
			printError("Registering trader network");
			traderInfoNetwork.initialize(this, sim);
		}
	}

	@Override
	public void handleMessage(MicroMessage message) {
		MicroMessage receivedMsg = message;
		if(receivedMsg.containsPerformative(MessageFields.ERROR_MESSAGE)){
			//only print messages that are not related to fully blacklisted agent pool
			if(!receivedMsg.containsKey(MTConnector.RANDOMCAST_ERROR_NUMBER_OF_AVAILABLE_TARGETS)){
				printError("Erroneous message: " + receivedMsg);
			}
		}
		
		if(MessageFactory.isTradeConfirmation(receivedMsg)){
			Float reportedValue = Float.parseFloat(receivedMsg.get(MessageFactory.TRADE_RESULT).toString());
			Float profit = null; 
			profit = reportedValue - MaghribiSim.GOODS_VALUE;
			String confirmationSender = receivedMsg.getSender();
			print("Received goods back from " + confirmationSender + " and earned " + profit);
			//memorize profit for this seller
			profitMemory.memorize(confirmationSender, profit);
			
			boolean correctDetection = false;
			boolean isCheater = false;
			boolean treated = false;
			if(MaghribiSim.cheatingDiscovery){		
				float detection = sim.random().nextFloat();
				// NO SUSPICION
				if(detection >= (1 - MaghribiSim.CHEATING_FALSE_DETECTION_PROBABILITY)){
					//wrong detection
					correctDetection = false;
				} else {
					//correct detection
					correctDetection = true;
				}
				//correctDetection = sim.random.nextBoolean(MaghribiSim.CHEATING_DISCOVERY_PROBABILITY);
				isCheater = Boolean.parseBoolean(receivedMsg.get(MessageFactory.CHEATING_INDICATOR).toString());
				print("Seller cheater? " + isCheater);
				Float cheatingAmount = Float.parseFloat(receivedMsg.get(MessageFactory.CHEATING_EXTENT).toString());
				//check for new negative cheating record
				if(cheatingAmount * -1 < mostNegativeCheating){
					mostNegativeCheating = cheatingAmount * -1;
				}
				//reset profit influence
				profit = 1f;
				if((correctDetection && isCheater) || (MaghribiSim.allowFalsePositiveCheatingDetection && (!correctDetection && !isCheater))){
					//if correctly detected, or if false positive detection possible (wrong detection of non-cheater) --> treat as cheater)
					//CHEATER DETECTED
					print("Think that agent " + confirmationSender + " is cheating (Current employees: " + employerWageList.keySet().toString() + ").");
					if(correctDetection && isCheater){
						Stats.detectedCheaterCorrectly();
					} else {
						Stats.falseDetectionOfCheater();
					}
					treatAsCheater(confirmationSender, cheatingAmount);
					treated = true;
				} else {
					if(MaghribiSim.allowFalsePositiveCheatingDetection){
						//consider only cross-overs
						if((correctDetection && !isCheater) || (!correctDetection && isCheater)){
							treatAsNonCheater(confirmationSender, profit);
							payWage(confirmationSender, 1.0f);
							treated = true;
						}
					} else {
						//NO CHEATING - or at least not suspected
						treatAsNonCheater(confirmationSender, profit);
						payWage(confirmationSender, 1.0f);
						treated = true;
						
					}
				}
				
				if(!treated){
					printError("Agent " + confirmationSender + " has not been treated - should never happen!");
				}
			} else {	
				//if no cheating detection activated, simply pay wage
				payWage(confirmationSender, 1.0f);
			}
			
			//indicate that agent does not have goods anymore + check for forced separation --> firing although not cheated
			if(employerWageList.containsKey(confirmationSender)){
				noGoodsWithEmployee(confirmationSender);
			}
			return;
		}
		
		//cheater reported by somebody else
		if(MaghribiSim.cheatingCommunication && MessageFactory.isCheaterInfo(receivedMsg)){
			String cheater = receivedMsg.get(MessageFactory.CHEATER_KEY).toString();
			print(receivedMsg.getSender() + " reported cheater " + cheater);
			//check if cheater is trader - then remember him
			if(MaghribiSim.oneOffTrades){
				print("Learned about other cheating trader " + cheater + " from " + receivedMsg.getSender() + " - Testing whether to consider that....");
				//if sender is of good memory, take his advice
				if(advisorMemory.containsKey(receivedMsg.getSender()) && advisorMemory.getValueForKey(receivedMsg.getSender()) > 0){
					print("Trusting " + receivedMsg.getSender() + ", so accepting advice.");
					if(cheater.startsWith(MaghribiSim.TRADER_PREFIX)){
						advisorMemory.memorize(cheater, MaghribiSim.CHEATER_VALENCE_OTHERS);
					} else if(cheater.startsWith(MaghribiSim.AGENT_PREFIX)){
						cheaterMemory.memorize(cheater, MaghribiSim.CHEATER_VALENCE_OTHERS);
					}
				} else if(advisorMemory.containsKey(receivedMsg.getSender()) && advisorMemory.getValueForKey(receivedMsg.getSender()) < 0){//else is sending agent is unknown, make acceptance dependent on truthfulness prob.
					print("Have bad experience with advisor " + receivedMsg.getSender() + ", so not taking his advice.");
				} else {
					print("Don't know advisor " + receivedMsg.getSender() + "...");
					print("... ignoring advice, don't trust enough (trust level: " + adviseTruthfullyProbability);
				}
				return;
			}
			if(MaghribiSim.cheatingMemory){
				cheaterMemory.memorize(cheater, MaghribiSim.CHEATER_VALENCE_OTHERS);
			}
			//add sender to information network - reciprocity
			if(useTraderInformationNetwork && MaghribiSim.reciprocalTraderInformationNetwork){
				enterTraderRelationship(receivedMsg.getSender());
			}
		}
		//non-cheater reported
		if(MaghribiSim.cheatingCommunication && MaghribiSim.communicatePositiveTransactions 
				&& MessageFactory.isNonCheaterInfo(receivedMsg)){
			String nonCheater = receivedMsg.get(MessageFactory.CHEATER_KEY).toString();
			print(receivedMsg.getSender() + " reported non-cheater " + nonCheater);
			if(MaghribiSim.oneOffTrades && nonCheater.startsWith(MaghribiSim.TRADER_PREFIX)){
				throw new RuntimeException("Learned positive experience about other cheating trader " + nonCheater + " - not yet handled.");
			} else {
				if(MaghribiSim.cheatingMemory){
					cheaterMemory.memorize(nonCheater, MaghribiSim.POSITIVE_VALENCE_OTHERS);
				}
				//add sender to information network
				if(useTraderInformationNetwork && MaghribiSim.reciprocalTraderInformationNetwork){
					enterTraderRelationship(receivedMsg.getSender());
				}
			}
		}
	}

	
	/**
	 * Treat agent as cheater.
	 * @param confirmationSender agent to be treated
	 * @param cheatingExtent amount of money trader has been cheated by (as positive value)
	 */
	private void treatAsCheater(String confirmationSender, float cheatingExtent){
		if(MaghribiSim.cheatingMemory){
			//negate cheating extent - it is positive number
			print("Remembered negative experience with seller " + confirmationSender);
			cheaterMemory.memorize(confirmationSender, MaghribiSim.CHEATER_VALENCE_OWN);
		}
		if(MaghribiSim.oneOffTrades && MaghribiSim.cheatingMemory){
			if(candidateAdvisorMap.containsKey(confirmationSender)){
				print("Associating cheating of seller with advice " + candidateAdvisorMap.get(confirmationSender));
				//test advice for correctness
				Advice advice = candidateAdvisorMap.get(confirmationSender);
				Boolean goodAdvice = null;
				if(advice.getAdviceValue() < 0){
					//if advisor was right - remember him as positive
					print("Advisor was right in giving me negative advice on " + confirmationSender);
					advisorMemory.memorize(advice.getAdvisor(), MaghribiSim.POSITIVE_VALENCE_OWN);
					Stats.gotGoodAdvice();
					if(MaghribiSim.adjustAdviceProbabilities){
						adjustAdviceProbabilities(1f);
					}
					goodAdvice = true;
				} else if(advice.getAdviceValue() > 0){
					//if advisor was wrong - remember him as betrayer
					print("Advisor betrayed me by giving me positive advice on " + confirmationSender);
					advisorMemory.memorize(advice.getAdvisor(), MaghribiSim.CHEATER_VALENCE_OWN);
					Stats.gotBadAdvice();
					if(MaghribiSim.adjustAdviceProbabilities){
						adjustAdviceProbabilities(-1f);
					}
					goodAdvice = false;
				}
				candidateAdvisorMap.remove(confirmationSender);
				if(MaghribiSim.cheatingCommunication && !goodAdvice){
					//advise other traders about misdoing of specific trader
					MicroMessage message = MessageFactory.makeCheaterInfo(advice.getAdvisor());
					traderInfoNetwork.shareCheaterInformation(message, MaghribiSim.maxAdviceRequestsForOneOffTrades);
				}
				
				//save advisor if fixed relations activated (and enough space left)
				if(goodAdvice != null){
					checkForAdditionToTraderConnections(goodAdvice, advice.getAdvisor());
				}
			}
			
			if(MaghribiSim.cheatingCommunication){
				MicroMessage mess = MessageFactory.makeCheaterInfo(confirmationSender);
				traderInfoNetwork.shareCheaterInformation(mess, MaghribiSim.maxAdviceRequestsForOneOffTrades);
			}
			
			return;
		}
		boolean treated = false;
		if(!treated){
			//default: just firing if no institution growing
			print("Paid no wage because of cheating - no attitude yet (or communication among cluster members not activated), thus always firing.");
			payWage(confirmationSender, 0.0f);
		}
		//fuzzycast instead of broadcast
		if(MaghribiSim.cheatingCommunication){
			if(useTraderInformationNetwork){
				sendFuzzycast(MessageFactory.makeCheaterInfo(confirmationSender), 1.0f, false, true, traderConnections);
			} else {
				ArrayList<String> inclusionList = new ArrayList<String>(Arrays.asList(SystemAgentLoader.findAgentNamesByRole(Trader.class)));
				sendFuzzycast(MessageFactory.makeCheaterInfo(confirmationSender), MaghribiSim.COMMUNICATION_FUZZINESS_QUOTA, false, true, inclusionList);
			}
		}
	}
	
	/**
	 * Treat agent as non-cheater
	 * @param confirmationSender agent to be treated as non-cheater
	 */
	private void treatAsNonCheater(String confirmationSender, Float profitExtent){
		//remember positive experience
		if(MaghribiSim.rememberPositiveTransactions){
			//remember positive experience
			cheaterMemory.memorize(confirmationSender, MaghribiSim.POSITIVE_VALENCE_OWN);// * profitExtent);
			print("Remembered positive experience with agent " + confirmationSender);
		}
		if(MaghribiSim.oneOffTrades && MaghribiSim.rememberPositiveTransactions){
			if(candidateAdvisorMap.containsKey(confirmationSender)){
				print("Attribute positive experience with seller with advice " + candidateAdvisorMap.get(confirmationSender));
				//test advice for correctness
				Advice advice = candidateAdvisorMap.get(confirmationSender);
				Boolean goodAdvice = null;
				if(advice.getAdviceValue() > 0){
					//if advisor was right - remember him as positive
					print("Advisor was right in giving me positive advice on " + confirmationSender);
					advisorMemory.memorize(advice.getAdvisor(), MaghribiSim.POSITIVE_VALENCE_OWN);
					Stats.gotGoodAdvice();
					if(MaghribiSim.adjustAdviceProbabilities){
						adjustAdviceProbabilities(1f);
					}
					goodAdvice = true;
				} else if(advice.getAdviceValue() < 0){
					//if advisor was wrong - remember him as betrayer
					print("Advisor betrayed me by giving me negative advice on " + confirmationSender);
					advisorMemory.memorize(advice.getAdvisor(), MaghribiSim.CHEATER_VALENCE_OWN);
					Stats.gotBadAdvice();
					if(MaghribiSim.adjustAdviceProbabilities){
						adjustAdviceProbabilities(-1f);
					}
					goodAdvice = false;
				}
				candidateAdvisorMap.remove(confirmationSender);
				if(goodAdvice != null){
					//save advisor if fixed relations activated (and enough space left)
					checkForAdditionToTraderConnections(goodAdvice, advice.getAdvisor());
				}
			}
			return;
		}
		//communicate positive experience
		if(MaghribiSim.cheatingCommunication && MaghribiSim.communicatePositiveTransactions){
			print("Communicated positive experience with agent " + confirmationSender);
			ArrayList<String> inclusionList = new ArrayList<String>(Arrays.asList(SystemAgentLoader.findAgentNamesByRole(Trader.class)));
			if(useTraderInformationNetwork){
				sendFuzzycast(MessageFactory.makeNonCheaterInfo(confirmationSender), 1.0f, false, true, traderConnections);
			} else {
				sendFuzzycast(MessageFactory.makeNonCheaterInfo(confirmationSender), MaghribiSim.COMMUNICATION_FUZZINESS_QUOTA, false, true, inclusionList);
			}
		}
	}
	
	private void payWage(String confirmationSender, Float ratioOfFullWage){
		if(MaghribiSim.oneOffTrades){
			return;
		}
	}
	
	@Inspect
	private HashMap<String, Boolean> goodsWithEmployees = new HashMap<String, Boolean>();
	
	
	private void noGoodsWithEmployee(String employee){
		if(employerWageList.containsKey(employee)){
			goodsWithEmployees.put(employee, false);
		} else {
			goodsWithEmployees.remove(employee);
		}
	}
	
	public void setupInitialTraderConnections(int numberOfConnections){
		if(MaghribiSim.NO_OF_TRADERS - 1 < numberOfConnections){
			printError("Too little agents to make " + numberOfConnections + " information network connections. Reduced to number of available agents.");
			numberOfConnections = MaghribiSim.NO_OF_TRADERS - 1;
		}
		//fill up initial trader connections
		ArrayList<String> candidateList = new ArrayList<String>(Arrays.asList(SystemAgentLoader.findAgentNamesByRole(Trader.class)));
		while(traderConnections.size() < numberOfConnections){
			String cand = candidateList.get(sim.random().nextInt(candidateList.size()));
			enterTraderRelationship(cand);
		}
	}
	
	public void enterTraderRelationship(String otherTrader){
		if(!traderConnections.contains(otherTrader) && !this.me().equals(otherTrader)){
			traderConnections.add(otherTrader);
		}
	}


	@Override
	public Float getPlaneWeight(String plane) {
		return 0f;
	}


	@Override
	public Float getTagWeight(Tag tag) {
		return 0f;
	}
	
	
	/**
	 * Checks if an agent should be added to trader connections. 
	 * @param goodAdvice Indicator if advice is good
	 * @param advisor Advisor
	 */
	private void checkForAdditionToTraderConnections(boolean goodAdvice, String advisor){
		if((MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_FIXED)
			|| MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_FIXED_MUTUAL))	
			&& goodAdvice
			&& traderConnections.size() < MaghribiSim.maxNumberOfTraderTrustRelations
			&& !traderConnections.contains(advisor)){
				if(MaghribiSim.traderInfoNetworkType.equals(NetworkType.NETWORK_FIXED_MUTUAL)){
					//check first if the added me, then add him
					if(((Trader)sim.agentDirectory.get(advisor)).addedYou(me())){
						traderConnections.add(advisor);
					}
				} else {
					//just add new advisor without concern for mutuality
					//printError("Added somebody as advisor: " + advice.getAdvisor());
					traderConnections.add(advisor);
				}
		}
	}

	@Override
	public Advice getAdvice(Advice advice) {
		advice.validateRequest();
		if(advice.getSender().equals(me())){
			printError("Requested advise from myself! Error!");
		}
		if(cheaterMemory.containsKey(advice.getAdviceObject())){
			Float truthfulProb = adviseTruthfullyProbability;
			//increased trust for known counterpart?
			if(MaghribiSim.increasedTrustProbabilityForTraderInTraderConnections){
				if(traderConnections != null && traderConnections.contains(advice.getSender())){
					//increase probability for truthful reporting if I know the requester
					truthfulProb += MaghribiSim.trustIncreaseForKnownTraders;
					print("Increased probability for truthful answer for known trader " + advice.getSender());
				} else {
					print("No probability for correct advice increase for unknown trader " + advice.getSender());
				}
			}
			boolean adviceDefined = false;
			//correct advice for positively remembered trader?
			if(MaghribiSim.correctAdviceToPositivelyRememberedAdvisors){
				//either positive memory of sender
				if((advisorMemory != null && advisorMemory.containsKey(advice.getSender()) && advisorMemory.getValueForKey(advice.getSender()) > 0)
						//or sender is contained in trader connections	
						|| (traderConnections != null && traderConnections.contains(advice.getSender()))
						){
					advice.setAdviceValue(cheaterMemory.getValueForKey(advice.getAdviceObject()));
					adviceDefined = true;
					print("GIVE ADVICE: Will send correct advice to positively remembered trader " + advice.getSender());
				}
			}
			//advise according to truthfulness
			if(!adviceDefined){
				//if not truthfully reporting to positively remembered requester, choose randomly
				//correct if it is too high or too low
				truthfulProb = truthfulProb > 1.0f ? 1.0f : truthfulProb < 0.0f ? 0.0f : truthfulProb;
				if(sim.random().nextBoolean(truthfulProb)){
					//tell the truth
					print("GIVE ADVICE: Advising " + advice.getSender() + " correctly, based on probability " + truthfulProb);
					advice.setAdviceValue(cheaterMemory.getValueForKey(advice.getAdviceObject()));
				} else {
					//lie
					print("GIVE ADVICE: Lied to " + advice.getSender() + ", based on probability " + truthfulProb);
					advice.setAdviceValue(cheaterMemory.getValueForKey(advice.getAdviceObject()) * -1);
				}
			}
		} else {
			print("Don't know anything about advice object " + advice.getAdviceObject());
		}
		advice.setAdvisor(me());
		return advice;
	}


	@Override
	public boolean addedYou(String otherAgent) {
		if(!traderConnections.contains(otherAgent)
				&& traderConnections.size() < MaghribiSim.maxNumberOfTraderTrustRelations){
			traderConnections.add(otherAgent);
		}
		return traderConnections.contains(otherAgent);
	}


	@Override
	public boolean removedYou(String otherAgent) {
		return traderConnections.remove(otherAgent);
	}
	
}
