package fuzzyMaghribis.network;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Set;
import org.graphstream.algorithm.generator.BarabasiAlbertGenerator;
import org.graphstream.algorithm.generator.WattsStrogatzGenerator;
import org.graphstream.algorithm.measure.ClosenessCentrality;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.SingleGraph;
import fuzzyMaghribis.MaghribiSim;
import static org.graphstream.algorithm.Toolkit.*;

public class GraphGenerators{
	
	private static final boolean debug = false;
	
	protected static Graph graph = null;
	
	/**
	 * Map containing node names and connection modifiers to be called to perform actual reassignment of relationships on agents.
	 */
	private static LinkedHashMap<String, ConnectionModifier> nodes = new LinkedHashMap<String, ConnectionModifier>();

	/**
	 * Adds individual modifier.
	 * @param modifier
	 */
	public static void defineNode(ConnectionModifier modifier){
		nodes.put(String.valueOf(nodes.size()), modifier);
	}
	
	/**
	 * Defines node set to be used in graph.
	 * @param nodeSet
	 */
	public static void defineNodes(Set<ConnectionModifier> nodeSet){
		for(ConnectionModifier modifier: nodeSet){
			nodes.put(String.valueOf(nodes.size()), modifier);
		}
	}
	
	/**
	 * Modifies actual connection information on registered agents.
	 * @param graph
	 */
	private static void modifyConnections(Graph graph){
		if(graph.getNodeCount() != nodes.size()){
			String message = "Size of nodes and generated graph nodes differ: (Initialized: " + nodes.size() + ", Graph: " + graph.getNodeCount() + ")";
			throw new RuntimeException(message);
			//System.err.println(message);
		}
		System.out.println("Number of links in graph: " + graph.getEdgeCount());
		for(int i=0; i<graph.getNodeCount(); i++){
			//graph node
			Node node = graph.getNode(i);
			//translated modifier
			ConnectionModifier modifier = nodes.get(node.toString());
			if(modifier == null){
				throw new RuntimeException("Discrepancy between generated nodes and initialized nodes: generated: " + node);
			}
			if(debug){
				System.out.println("Node: " + node);
				System.out.println("Modifier for node: " + modifier);
			}
			ArrayList<String> relatedNodes = new ArrayList<>();
			for(Edge edge: node.getEdgeSet()){
				Node otherNode = (edge.getNode0().equals(node) ? edge.getNode1() : edge.getNode0());
				if(debug){
					System.out.println("Outgoing Edge for " + modifier.getName() + ": " + edge);
					System.out.println("Looking up " + otherNode.toString());
					System.out.println("translated to: " + nodes.get(otherNode.toString()).getName());
				}
				relatedNodes.add(nodes.get(otherNode.toString()).getName());
			}
			try{
				modifier.modifyConnections(relatedNodes);
			} catch(NullPointerException e){
				System.err.println("Node " + modifier.getName()  + " makes trouble");
			}
		}
	}
	
	/**
	 * Generates graph based on Watts Strogatz algorithm - small world.
	 * @param maxNumberOfRelations maximum number of relations between nodes
	 * @param seed Seed for RNG
	 * @param display true -> display result
	 */
	public static void generateWattsStrogatzGraph(int maxNumberOfRelations, long seed, boolean display){
		graphInitializationCheck();
		System.err.println("Reassigning connections based on Watts Strogatz algorithm for " + nodes.size() + " nodes.");
		graph = new SingleGraph("Small world");
		WattsStrogatzGenerator smallWorld = new WattsStrogatzGenerator(nodes.size(), maxNumberOfRelations, MaghribiSim.wattsStrogatzRewiringProbability);
		smallWorld.setRandomSeed(seed);
		smallWorld.addSink(graph);
		smallWorld.begin();
		while(smallWorld.nextEvents()){};
		smallWorld.end();
		modifyConnections(graph);
		if(display){
			graph.display(false);
		}
	}
	
	/**
	 * Generates graph based on Barabasi Albert algorihm - scale free
	 * @param maxLinksPerStep Maximum number of edges created when a new node is added.
	 * @param seed Seed for RNG
	 * @param display true -> display result
	 */
	public static void generateBarabasiAlbertGraph(int maxLinksPerStep, long seed, boolean display){
		graphInitializationCheck();
		System.err.println("Reassigning connections based on Barabasi Albert algorithm for " + nodes.size() + " nodes.");
		graph = new SingleGraph("Scale free");
		BarabasiAlbertGenerator scaleFree = new BarabasiAlbertGenerator(maxLinksPerStep);
		scaleFree.setRandomSeed(seed);
		scaleFree.addSink(graph);
		scaleFree.begin();
		//one less iteration over nodes as one is added anyway
		for(int i=0; i<nodes.size() - 2; i++) {
			scaleFree.nextEvents();
		}
		scaleFree.end();
		modifyConnections(graph);
		if(display){
			graph.display();
		}
	}
	
	/**
	 * Checks if graph has already been initialized. 
	 * Throws exception if so.
	 */
	private static void graphInitializationCheck(){
		if(graph != null){
			throw new RuntimeException("Graph has already been initialized. Can only be done once.");
		}
	}
	
	/**
	 * Resets Graph generator to allow new generation.
	 */
	protected static void resetGraphGenerator(){
		System.out.println("Reset graph generator.");
		graph = null;
		nodes.clear();
	}

	public static void main(String args[]) {
		
		int number = 200;
		
		//Playground for testing new algorithms
		
		Graph graph2 = new SingleGraph("Scale free");
		BarabasiAlbertGenerator scaleFree = new BarabasiAlbertGenerator(3);
		scaleFree.addSink(graph2);
		scaleFree.begin();
		for(int i=0; i<number; i++) {
			scaleFree.nextEvents();
		}
		scaleFree.end();
		graph2.display();
		
		DegreesAlgorithm alg = new DegreesAlgorithm();
		alg.init(graph2);
		alg.compute();
		System.out.println("Graph measures: " + alg);
		
		ClosenessCentrality cc = new ClosenessCentrality();
		cc.init(graph2);
		cc.compute();
		//System.out.println(cc.cc.getCentralityAttribute());
		
		System.out.println("Avg. cluster coefficient: " + averageClusteringCoefficient(graph2));
		
		
		/*for(int i=0; i<number; i++){
			Node node = graph2.getNode(i);
			System.out.println("Node: " + node);
			for(Edge edge: node.getEdgeSet()){
				Node otherNode = (edge.getNode0().equals(node) ? edge.getNode1() : edge.getNode0());
				
				System.out.println("Outgoing Edge for " + node.getId() + ": " + edge);
				System.out.println("Other node: " + otherNode);
			}
		}*/
		
		
		
		
	}


}